;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xb1ca3db7038f469fb94f4ac37d8e444f . "./code.cg2js/cg-common-control.fasl")
(#x6d3e9023fd003666c7bc38437993f07e . "./code.cg2js/cg-clipboard-stack.fasl")
(#xf2a54b59876d507b2b24795b9ef14850 . "./code.cg2js/cg-yes-no-string.fasl")
(#x7f512e88c933dc62ee77cead26cea3f4 . "./code.cg2js/cg-clipboard.fasl")
(#xf9c8fed60bc671dfa28c8e457c5a95da . "./code.cg2js/cg-dde-utils.fasl")
(#x9339d20752f1a60defed085b353ac9e3 . "./code.cg2js/cg-editable-text.fasl")
(#x59e655328870c668cba1ff7d522627e5 . "./code.cg2js/cg-player.fasl")
(#x76cfb0b299a4a4f3bb37bf7e3940821f . "./code.cg2js/cg-text-or-combo.fasl")
(#xaa67676caaa9ccd41b9212794aca0b70 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xc22fd4b0cf4eb39e12f3808fec38668b . "./code.cg2js/cg-graphics-context.fasl")
(#xa8c4709231cc0164dc087a9072e8b1c9 . "./code.cg2js/cg-choose-printer.fasl")
(#x51d8a580028e595bbbb1fe4763f76093 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x22e2a436602c21940a2a7b9379aa85b9 . "./code.cg2js/cg-edit-in-place.fasl")
(#x836927186b670f0277d13d0651e89e47 . "./code.cg2js/cg-split-bar.fasl")
(#x1da9a62a4a169ca7d8747d6c76de00ac . "./code.cg2js/cg-timer.fasl")
(#xb91a6e6e115b629c9bbf2b62298da8bd . "./code.cg2js/cg-progress-indicator.fasl")
(#x64ba2c5e18c276f8b8afaa7afb290e50 . "./code.cg2js/cg-object-editor.fasl")
(#x33e937cac146f6df870c039b68863378 . "./code.cg2js/cg-fill-texture.fasl")
(#xd6dc023023f0ed5bd2dfbf1d8639ad27 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xc4292ec1d2da587c4519ee246fbfe230 . "./code.cg2js/cg-os-window.fasl")
(#x6e0148c58b598d0a1b7f9933bbe464f9 . "./code.cg2js/cg-color-dialog.fasl")
(#xff1be1ce559670d5a45a801159ef6be3 . "./code.cg2js/cg-combo-box.fasl")
(#xbf812f59049e6f296034b83102cc1045 . "./code.cg2js/cg-directory-dialog.fasl")
(#x7042b98bd61b52bd962a218fdefd740f . "./code.cg2js/cg-tooltip.fasl")
(#x28a7dd3fe6385be8b461e0fe73123998 . "./code.cg2js/cg-lettered-menu.fasl")
(#xa9c1987d59ea1effb9ae8bb374cd9e50 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x395ac441b60c530045d38a94cea75556 . "./code.cg2js/cg-pixmap.fasl")
(#x27ad0f4a142159a8ba74c8de146b83df . "./code.cg2js/cg.fasl")
(#x050fb499d6dce49aefe4490df1d84d00 . "./code.cg2js/cg-class-slot-grid.fasl")
(#xb7fc0211aa30b0d70232827c8c4a8df9 . "./code.cg2js/cg-template-string.fasl")
(#xdbbb3150b12d21c4a1dd969d5a2ddbca . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x65419b8d94a1dffd34bf152991e8099d . "./code.cg2js/cg-check-box.fasl")
(#x84042ccc0cea3b9c613a6378a73cad35 . "./code.cg2js/cg-toggling-widget.fasl")
(#x47b9f02f6d17f321b24742e14c4747ad . "./code.cg2js/cg-radio-button.fasl")
(#xf7c25749e91fa6d7f44d0d3fd7564efe . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xc94e9cd1cf928d81463d1a38f117f5a0 . "./code.cg2js/cg-scaling-stream.fasl")
(#xa1ba94ef7b990c91059b16f91606fe30 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xe466a0807a881e05b46d10aa75990a1e . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xc4f9eddc22fa7caae06d3234ae9adaac . "./code.cg2js/cg-grid-widget.fasl")
(#xf64919a04c4469b446f34e3793a7b740 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x67be412a34fde8201cd053a9dbe8d165 . "./code.cg2js/cg-item-list.fasl")
(#xec1b7f9859f9cfe30bf43992ecab7b6d . "./code.cg2js/cg-paren-matching.fasl")
(#xad3325dd877371bf9f3966f67d21ef74 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x71b0a9f16dfc024557d96494e616e8dd . "./code.cg2js/cg-html-dialog.fasl")
(#x9a9aab514da6599c4ee9ff8f7dcdaccd . "./code.cg2js/cg-pixmap-widget.fasl")
(#x09454ff416fab512cf69d8632bc54c69 . "./code.cg2js/cg-project-window.fasl")
(#x7310acdd76c16b91f6570a2895a31e00 . "./code.cg2js/cg-toolbar.fasl")
(#x02f7c71c070335122e4daa2b010980fe . "./code.cg2js/cg-caret.fasl")
(#xe377e4603546da0ce2d2d81bdb4cbb6c . "./code.cg2js/cg-scroll-bar.fasl")
(#x81cc6e0877ed290508611126db1c32ed . "./code.cg2js/cg-web-browser.fasl")
(#xa1b11794e7b18021232a669a17bab77d . "./code.cg2js/cg-dropping-outline.fasl")
(#xec38f8c73c0c365ed78d85ba18e5b36b . "./code.cg2js/cg-font-dialog.fasl")
(#xd8e4bf2666e40004df6bd70a501cabc3 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xe22424dbfe36966c7c9eb4b03431d97f . "./code.cg2js/cg-yes-no-list.fasl")
(#x94e2cdd9ba5cd8aff97da963030cae5e . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xf43427f6e55823bd52f395df7ea7e905 . "./code.cg2js/cg-lamp.fasl")
(#x811a38b7ffff11b75b2c52109cb8af47 . "./code.cg2js/cg-drawable.fasl")
(#x6ec8def5064c56abad803b622cc1ae1b . "./code.cg2js/cg-choice-list.fasl")
(#x737ef296ab05be7d7147b3861b25b66d . "./code.cg2js/cg-curve.fasl")
(#xf740baee166faa0dcb6976670138eb36 . "./code.cg2js/cg-base.fasl")
(#x163f221c300e0c897f6e294612303075 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x38463d0bcf49fc9c603a72b216ee447e . "./code.cg2js/cg-printing.fasl")
(#x474f4bef7c9c3d0d402baf93b23714c4 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xe72add95fd8d20ddceee706c0d2bedea . "./code.cg2js/cg-hotspot.fasl")
(#x728a92621a2b97179a4dc1ea0c6e57fb . "./code.cg2js/cg-object-editor-layout.fasl")
(#xe0437f1c51b61d3ba0e38dc0728b6023 . "./code.cg2js/cg-utility-dialog.fasl")
(#x8f64ce95e76be996c371708afcab3ada . "./code.cg2js/cg-shortcut-menu.fasl")
(#x0a12514d9e79787c21ba377c3d0cb126 . "./code.cg2js/cg-comtab.fasl")
(#x666c0b1c2e3f1832971d7db21ee5b59d . "./code.cg2js/cg-os-widget.fasl")
(#xad29228e023c070dcfdcbd987dfb9256 . "./code.cg2js/cg-icon.fasl")
(#x91d79d62e7810a1c40ee8b78926784f3 . "./code.cg2js/cg-dev.fasl")
(#x3a99a0bd1a0f3652d2f8ebe25782388b . "./code.cg2js/cg-calendar.fasl")
(#x1fe791b7670f8668c9609b7e749bf3cb . "./code.cg2js/cg-group-box.fasl")
(#xf58759c93def5d9485f498d24718d474 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x8e0fc17bc38f697f5c69ba31c324ee7d . "./code.cg2js/cg-bitmap-pane.fasl")
(#x6f0d305970fb0255ef875b72d145ce6e . "./code.cg2js/cg-drag-and-drop.fasl")
(#x03fc08276d7e140482662fb5a3d048c8 . "./code.cg2js/cg-nodes.fasl")
(#x3df46bd432aaab344b6d2aa24706cff8 . "./code.cg2js/cg-intersect.fasl")
(#xdf73fd2316818bad8791c5f46f64b749 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xeea8e364b7253e13737a7a22deb08be1 . "./code.cg2js/cg-class-support.fasl")
(#x596bea653ab8cdcfc1b31f0f3dd1662b . "./code.cg2js/cg-dialog-item.fasl")
(#x8e3c2337fa88efa1380fecbaeeb98188 . "./code.cg2js/cg-plot-widget.fasl")
(#x51571ce58272deb52ac54e94a505cda4 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x0f00cbe10b983cce79996daa1bf391a0 . "./code.cg2js/cg-up-down-control.fasl")
(#x470654378b53d760f4675e62849f99ff . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x67082a21d5b8d294d6b45cc53ad76677 . "./code.cg2js/cg-status-bar.fasl")
(#x9c3e1ca94527eda748e63570dca340be . "./code.cg2js/cg-class-grid.fasl")
(#x93af6116bf19af31079cfc917e61ba58 . "./code.cg2js/cg-wrap-string.fasl")
(#x8a7630c3fd3637c7b52c106e1abf8385 . "./code.cg2js/cg-layout.fasl")
(#x0c841c3827cb5bf0fabb2cdd41033eea . "./code.cg2js/cg-lisp-widget.fasl")
(#xc99e8a264c7596d18d6bd8f8cec055d0 . "./code.cg2js/cg-button.fasl")
(#x022921b6474c4937195f9791a4c2a256 . "./code.cg2js/cg-string-dialog.fasl")
(#x54ef18e1f5f861b4334b939c5828abf0 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xa1e7e0f9f67c8e6ec3fdd6aa3836e580 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xd73133c2626d2fa45de5da35cf374109 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x9002e465ab5fcf9f3b2773dbb36711b5 . "./code.cg2js/cg-static-text.fasl")
(#x88dbec713e5ae516675f8eb08d88d01b . "./code.cg2js/cg-outline.fasl")
(#x8b939d77cf444441408461f5f7593689 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x84be330c6dd26ce071f1f1850a3d9a8f . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xdc87b0ae9b201a5874246e7d01907d5a . "./code.cg2js/cg-picture-widget.fasl")
(#x536e043d1cd895c8eeaeba7fa055dd54 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x89b3f66fe008e28e712105fd0b342ddc . "./code.cg2js/cg-chart-widget.fasl")
(#x67bd504a7db93f0002ae1f967f376c22 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x4a107aac5704835d36bee21c10a8271b . "./code.cg2js/cg-lisp-text.fasl")
(#x45c548a956043d07a233758aeb52f0fb . "./code.cg2js/cg-palette.fasl")
(#x1df64f85c8d1cf0a62462df166935c0f . "./code.cg2js/cg-property.fasl")
(#xae7008b3c22880c46e661bb079b51a2d . "./code.cg2js/cg-intersect-posbox.fasl")
(#xd720e3d04c1c14c212f98a589551e597 . "./code.cg2js/cg-file-dialog.fasl")
(#xdabd6833f10b8256dcbf851778a77915 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xbdc9e7f91a6b01108c91fc7e0b692b27 . "./code.cg2js/cg-get-pixmap.fasl")
(#xd4274e0c2f269dd00b2d717d344ea8ee . "./code.cg2js/cg-tray.fasl")
(#xd21ad6b8992ce1754b30e633a5209026 . "./code.cg2js/cg-acache.fasl")
(#x66a24263e9a7e40b8418c0727e477788 . "./code.cg2js/cg-message-dialog.fasl")
(#x9d2d9be282cf60c99669abfeec1f4e8a . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xfcde4cc6830d234b2bc8873fbadbfb3d . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xb16c59e02c588cff574cac19fab5dcc3 . "./code.cg2js/cg-tab-control.fasl")
(#x6e62361e47149b639fb3b43c6ed95874 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x1ff801d7ca7cf6817e96ec658acac929 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x078f0d46e660014099401bea5c8943b1 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xc83051c0f5300da78f0781b56abeba1a . "./code.cg2js/cg-menu.fasl")
(#xae1053a6063e9f98af0b83252904ba62 . "./code.cg2js/cg-get-position.fasl")
(#xa46a4aad149ab0690e24053a6d99f59f . "./code.cg2js/cg-trackbar.fasl")
(#x57ac6757a1803cdb843f2ec68ae25c83 . "./code.cg2js/cg-selected-object.fasl")
(#xefb2a3e6092360297ee46fe503d7a717 . "./code.cg2js/cg-list-view.fasl")
(#x517d9c015f939dde5bab810c03ebf591 . "./code.cg2js/cg-html-widget.fasl")
(#x01d4ab30490e7788893d09999b795c6f . "./code.cg2js/cg-sample-file-menu.fasl")
(#x03c034288f97361834aea8ec82f5582f . "./code.cg2js/cg-text-widget.fasl")
(#xd1da4e0c23f175fd35af5a844c78ef45 . "./code.idejs/ide-base.fasl")
(#xece182522c5c24b1faeb1aa109ba69b2 . "./code.idejs/ide-process-browser.fasl")
(#xe16a3350cf5b02aee723000b1490bb0f . "./code.idejs/ide-patch-dialog.fasl")
(#x234e5e5bf998a302e7757a8f568d856e . "./code.idejs/ide-keyshort.fasl")
(#x61e06d775d5f0e8e04249eed20b75a15 . "./code.idejs/ide-window-list.fasl")
(#x9f1485969ffbf12ef4eb2a657d2e6fbc . "./code.idejs/ide.fasl")
(#x2a579e55ac18c29e0f1c7a445d44e154 . "./code.idejs/ide-editor.fasl")
(#x0c6c3f52ac249264e18e4049e88c83ec . "./code.idejs/ide-clipboard.fasl")
(#x3cea7900ea4dfdab58c5461679a0c128 . "./code.idejs/ide-trace.fasl")
(#xbdbb4bd8c03240edd90d5db8fad1d0c8 . "./code.idejs/ide-class-browser.fasl")
(#xde204af1ccbb944603cc32c99ea96f80 . "./code.idejs/ide-grapher.fasl")
(#x94324957d4ede87c639799cf19ab6cb6 . "./code.idejs/ide-apropos.fasl")
(#xd53bca1f37609dbab7a495c707212dbd . "./code.idejs/ide-navigator.fasl")
(#x589b4d19b262d09be1a0245f45e5007c . "./code.idejs/ide-find-definitions.fasl")
(#x62bc74efc0637bb22cdcfea279ef011c . "./code.idejs/ide-project.fasl")
(#xd4596b42c76371a13df97a92d2dba6ce . "./code.idejs/ide-class-grapher.fasl")
(#x21aa3cc6f4ff95f0cd446686b990166c . "./code.idejs/ide-acache.fasl")
(#x77cfa5f249f77a08ba9ef2bbd7871bb3 . "./code.idejs/ide-find-in-files.fasl")
(#x487c7a5da780176c8457759ea3ed6160 . "./code.idejs/ide-inspector.fasl")
(#xdddeadd6718c5e4cc21e395563219643 . "./code.idejs/ide-intro.fasl")
(#x305a3e6ce11bf21dc3eb9cdf23683097 . "./code.idejs/ide-package-list.fasl")
(#xf43cc5fd9f037fb331d4a2c923ec40d4 . "./code.idejs/ide-dev.fasl")
(#x47ef3439234c0d6fd5a71063c9c69d72 . "./code.idejs/ide-form.fasl")
(#x7e8a653f63c59e56c07642baa40b4a7f . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
